@@ -579,7 +579,7 @@ service_method_read_alias (GkdSecretService *self, DBusMessage *message)
 	const char *alias;
 	gchar *path = NULL;
 	const gchar *identifier;
-	GP11Object  *collection;
+	GP11Object  *collection = NULL;
 
 	if (!dbus_message_get_args (message, NULL, DBUS_TYPE_STRING, &alias, DBUS_TYPE_INVALID))
 		return NULL;
@@ -591,8 +591,9 @@ service_method_read_alias (GkdSecretService *self, DBusMessage *message)
 		path = gkd_secret_util_build_path (SECRET_COLLECTION_PREFIX, identifier, -1);
 
 	/* Make sure it actually exists */
-	collection = gkd_secret_objects_lookup_collection (self->objects,
-	                                                   dbus_message_get_sender (message), path);
+	if (path)
+		collection = gkd_secret_objects_lookup_collection (self->objects,
+		                                                   dbus_message_get_sender (message), path);
 	if (collection == NULL) {
 		g_free (path);
 		path = NULL;
@@ -830,7 +831,7 @@ service_message_handler (GkdSecretService *self, DBusMessage *message)
 		return service_method_lock (self, message);
 
 	/* org.gnome.keyring.Service.ChangeLock() */
-	if (dbus_message_is_method_call (message, INTERNAL_SERVICE_INTERFACE, "ChangeLock"))
+	if (dbus_message_is_method_call (message, SECRET_SERVICE_INTERFACE, "ChangeLock"))
 		return service_method_change_lock (self, message);
 
 	/* org.freedesktop.Secret.Service.ReadAlias() */
@@ -841,10 +842,6 @@ service_message_handler (GkdSecretService *self, DBusMessage *message)
 	if (dbus_message_is_method_call (message, SECRET_SERVICE_INTERFACE, "SetAlias"))
 		return service_method_set_alias (self, message);
 
-	/* org.gnome.keyring.InternalUnsupportedGuiltRiddenInterface.ChangeLock() */
-	if (dbus_message_is_method_call (message, INTERNAL_SERVICE_INTERFACE, "ChangeLock"))
-		return service_method_change_lock (self, message);
-
 	/* org.gnome.keyring.InternalUnsupportedGuiltRiddenInterface.CreateWithMasterPassword */
 	if (dbus_message_is_method_call (message, INTERNAL_SERVICE_INTERFACE, "CreateWithMasterPassword"))
 		return service_method_create_with_master_password (self, message);
