@@ -129,7 +129,8 @@ require_session_async (GP11Object *self, GP11Call *call,
 		run_call_with_session (call, session);
 		g_object_unref (session);
 	} else {
-		gp11_slot_open_session_async (data->slot, flags, cancellable, opened_session, call);
+		gp11_slot_open_session_async (data->slot, flags, NULL, NULL,
+		                              cancellable, opened_session, call);
 	}
 	
 }
@@ -668,7 +669,7 @@ free_set_attributes (SetAttributes *args)
  * gp11_object_set:
  * @self: The object to set attributes on.
  * @err: A location to return an error.
- * ...: The attributes to set.
+ * @...: The attributes to set.
  *
  * Set PKCS#11 attributes on an object.
  * This call may block for an indefinite period.
@@ -892,8 +893,8 @@ free_get_attributes (GetAttributes *args)
  * gp11_object_get:
  * @self: The object to get attributes from.
  * @err: A location to store an error.
- * ...: The attribute types to get.
- * 
+ * @...: The attribute types to get.
+ *
  * Get the specified attributes from the object. This call may
  * block for an indefinite period.
  * 
@@ -929,7 +930,7 @@ gp11_object_get (GP11Object *self, GError **err, ...)
 }
 
 /**
- * gp11_object_get:
+ * gp11_object_get_full:
  * @self: The object to get attributes from.
  * @attrs: The attributes to get, with the types filled in.
  * @cancellable: Optional cancellation object, or NULL.
@@ -1076,17 +1077,19 @@ perform_get_attribute_data (GetAttributeData *args)
 		return rv;
 	
 	/* Allocate memory for the value */
-	args->result = (args->allocator) (NULL, attr.ulValueLen);
+	args->result = (args->allocator) (NULL, attr.ulValueLen + 1);
 	g_assert (args->result);
 	attr.pValue = args->result;
-	
+
 	/* Now get the actual value */
 	rv = (args->base.pkcs11->C_GetAttributeValue) (args->base.handle, args->object,
 	                                               &attr, 1);
 
-	if (rv == CKR_OK)
+	if (rv == CKR_OK) {
 		args->n_result = attr.ulValueLen;
-	
+		args->result[args->n_result] = 0;
+	}
+
 	return rv;
 }
 
@@ -1105,8 +1108,10 @@ free_get_attribute_data (GetAttributeData *args)
  * @n_data: The length of the resulting data.
  * @err: A location to store an error.
  * 
- * Get the data for the specified attribute from the object. This call 
- * may block for an indefinite period.
+ * Get the data for the specified attribute from the object. For convenience
+ * the returned data has a null terminator.
+ *
+ * This call may block for an indefinite period.
  * 
  * Return value: The resulting PKCS#11 attribute data, or NULL if an error occurred. 
  **/
@@ -1129,8 +1134,10 @@ gp11_object_get_data (GP11Object *self, gulong attr_type, gsize *n_data, GError
  * @n_data: The length of the resulting data.
  * @err: A location to store an error.
  * 
- * Get the data for the specified attribute from the object. This call 
- * may block for an indefinite period.
+ * Get the data for the specified attribute from the object. For convenience
+ * the returned data has an extra null terminator, not included in the returned length.
+ *
+ * This call may block for an indefinite period.
  * 
  * Return value: The resulting PKCS#11 attribute data, or NULL if an error occurred. 
  **/
@@ -1182,8 +1189,9 @@ gp11_object_get_data_full (GP11Object *self, gulong attr_type, GP11Allocator all
  * @callback: Called when the operation completes.
  * @user_data: Data to be passed to the callback.
  * 
- * Get the data for the specified attribute from the object. This call will
- * return immediately and complete asynchronously.
+ * Get the data for the specified attribute from the object.
+ *
+ * This call will return immediately and complete asynchronously.
  **/
 void
 gp11_object_get_data_async (GP11Object *self, gulong attr_type, GP11Allocator allocator, 
@@ -1217,8 +1225,10 @@ gp11_object_get_data_async (GP11Object *self, gulong attr_type, GP11Allocator al
  * @n_data: The length of the resulting data.
  * @err: A location to store an error.
  *
- * Get the result of an operation to get attribute data from 
- * an object. 
+ * Get the result of an operation to get attribute data from
+ * an object. For convenience the returned data has an extra null terminator,
+ * not included in the returned length.
+ *
  * 
  * Return value: The PKCS#11 attribute data or NULL if an error occurred.
  **/
