@@ -840,10 +840,12 @@ gck_module_store_token_object (GckModule *self, GckTransaction *transaction, Gck
 	g_return_if_fail (GCK_IS_OBJECT (object));
 	g_assert (GCK_MODULE_GET_CLASS (self)->store_token_object);
 
-	if (gck_object_is_transient (object))
-		add_transient_object (self, transaction, object);
-	else
+	if (gck_object_is_transient (object)) {
+		if (g_hash_table_lookup (self->pv->transient_objects, object) == NULL)
+			add_transient_object (self, transaction, object);
+	} else {
 		GCK_MODULE_GET_CLASS (self)->store_token_object (self, transaction, object);
+	}
 }
 
 void
