@@ -47,16 +47,23 @@ struct _GckObjectClass {
 	GObjectClass parent_class;
 	
 	/* signals ------------------------------------------------------------------ */
-	
+
+	void (*expose_object) (GckObject *object, gboolean exposed);
+
 	void (*notify_attribute) (GckObject *object, CK_ATTRIBUTE_TYPE attr_type);
 	
 	/* virtual methods  --------------------------------------------------------- */
     
-	CK_RV (*get_attribute) (GckObject *object, CK_ATTRIBUTE *attr);
+	CK_RV (*get_attribute) (GckObject *object, GckSession *session,
+	                        CK_ATTRIBUTE *attr);
 	
-	void (*set_attribute) (GckObject *object, GckTransaction *transaction, CK_ATTRIBUTE *attr);
+	void (*set_attribute) (GckObject *object, GckSession *session,
+	                       GckTransaction *transaction, CK_ATTRIBUTE *attr);
 	
-	CK_RV (*unlock) (GckObject *self, CK_UTF8CHAR_PTR pin, CK_ULONG n_pin);
+	void (*create_attributes) (GckObject *object, GckSession *session,
+	                           GckTransaction *transaction, CK_ATTRIBUTE *attrs, CK_ULONG n_attrs);
+
+	CK_RV (*unlock) (GckObject *self, GckCredential *cred);
 };
 
 GType                  gck_object_get_type               (void);
@@ -66,41 +73,81 @@ CK_OBJECT_HANDLE       gck_object_get_handle             (GckObject *self);
 void                   gck_object_set_handle             (GckObject *self,
                                                           CK_OBJECT_HANDLE handle);
 
+GckModule*             gck_object_get_module             (GckObject *self);
+
 GckManager*            gck_object_get_manager            (GckObject *self);
 
 const gchar*           gck_object_get_unique             (GckObject *self);
 
-CK_RV                  gck_object_unlock                 (GckObject *self, 
-                                                          CK_UTF8CHAR_PTR pin, 
-                                                          CK_ULONG n_pin);
+gboolean               gck_object_is_token               (GckObject *self);
+
+gboolean               gck_object_is_transient           (GckObject *self);
+
+CK_RV                  gck_object_unlock                 (GckObject *self,
+                                                          GckCredential *cred);
+
+void                   gck_object_destroy                (GckObject *self,
+                                                          GckTransaction *transaction);
+
+gboolean               gck_object_is_exposed             (GckObject *self);
+
+void                   gck_object_expose                 (GckObject *self,
+                                                          gboolean expose);
+
+void                   gck_object_expose_full            (GckObject *self,
+                                                          GckTransaction *transaction,
+                                                          gboolean expose);
 
 gboolean               gck_object_match                  (GckObject *self,
+                                                          GckSession *session,
                                                           CK_ATTRIBUTE_PTR attr);
 
 gboolean               gck_object_match_all              (GckObject *self,
+                                                          GckSession *session,
                                                           CK_ATTRIBUTE_PTR attrs,
                                                           CK_ULONG n_attrs);
 
 CK_RV                  gck_object_get_attribute          (GckObject *self,
+                                                          GckSession *session,
                                                           CK_ATTRIBUTE_PTR attr);
 
 void                   gck_object_set_attribute          (GckObject *self,
+                                                          GckSession *session,
                                                           GckTransaction *transaction,
                                                           CK_ATTRIBUTE_PTR attr);
 
+void                   gck_object_create_attributes      (GckObject *self,
+                                                          GckSession *session,
+                                                          GckTransaction *transaction,
+                                                          CK_ATTRIBUTE_PTR attrs,
+                                                          CK_ULONG n_attrs);
+
 void                   gck_object_notify_attribute       (GckObject *self,
                                                           CK_ATTRIBUTE_TYPE attr_type);
 
 gboolean               gck_object_get_attribute_boolean  (GckObject *self,
+                                                          GckSession *session,
                                                           CK_ATTRIBUTE_TYPE type,
                                                           gboolean *value);
 
 gboolean               gck_object_get_attribute_ulong    (GckObject *self,
+                                                          GckSession *session,
                                                           CK_ATTRIBUTE_TYPE type,
                                                           gulong *value);
 
 void*                  gck_object_get_attribute_data     (GckObject *self,
+                                                          GckSession *session,
                                                           CK_ATTRIBUTE_TYPE type,
                                                           gsize *n_data);
 
+gboolean               gck_object_has_attribute_ulong    (GckObject *self,
+                                                          GckSession *session,
+                                                          CK_ATTRIBUTE_TYPE type,
+                                                          gulong value);
+
+gboolean               gck_object_has_attribute_boolean  (GckObject *self,
+                                                          GckSession *session,
+                                                          CK_ATTRIBUTE_TYPE type,
+                                                          gboolean value);
+
 #endif /* __GCK_OBJECT_H__ */
